<?php

/**
 * Implement hook_drush_command().
 */
function modulate_drush_command() {
  $items = array();

  $items['modulate'] = array(
    'callback' => 'modulate_drush_generate',
    'description' => 'Generates the module.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_DATABASE,
  );
  $items['modulate-cleanup'] = array(
    'callback' => 'modulate_drush_cleanup',
    'description' => 'Removes the generated items from the database',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_DATABASE,
  );
  return $items;
}

function modulate_drush_generate() {
  $result = db_query('SELECT theme, module, delta, pages FROM {blocks} WHERE visibility = 2 AND status = 1');
  while ($block = db_fetch_object($result)) {
    $function_name = '_modulate_' . $block->theme . '_' . $block->module . '_' . str_replace('-', '_', $block->delta);
    $function = "function $function_name() {";
    $function .= preg_replace('/^\s*<\?(php)?(.*)\?>$/s', '\2', $block->pages);
    $function .= "\n}";
    $functions[] = $function;
    $conditions[] = array(
      'theme' => $block->theme,
      'module' => $block->module,
      'delta' => $block->delta,
      'function_name' => $function_name,
    );
  }
  $conditions = var_export($conditions, TRUE);
  $functions = implode("\n\n", $functions);
  $module = <<<MODULE
<?php

/**
 * Implements hook_db_rewrite_sql().
 */
function modulate_db_rewrite_sql(\$query, \$primary_table, \$primary_field, \$args) {
  global \$theme_key;
  if (\$primary_table == 'b' && \$primary_field == 'bid') {
    \$where = array();
    \$conditions = $conditions;
    foreach (\$conditions as \$condition) {
      \$function = \$condition['function_name'];
      if (\$condition['theme'] == \$theme_key && !\$function()) {
        \$where[] = "NOT (b.module = '". \$condition['module'] . "' AND b.delta = '" . \$condition['delta'] . "') ";
      }
    }
    if (\$where) {
      return array('where' => implode(' AND ', \$where));
    }
  }
}

$functions
MODULE;
  file_put_contents(dirname(__FILE__) . '/modulate.module', $module);
}

function modulate_drush_cleanup() {
  db_query("UPDATE {blocks} SET visibility = 0, pages = '' WHERE visibility = 2 AND status = 1");
}
